# -*- coding: iso-8859-1 -*-
# -----------------------------------------------------------------------
# rssperiodic.py - This is the Freevo RSS feed module
# -----------------------------------------------------------------------
# $Id: rssperiodic.py 9514 2007-04-29 08:32:30Z duncan $
#
# Notes:
# Todo:        
#
# -----------------------------------------------------------------------
# Freevo - A Home Theater PC framework
# Copyright (C) 2002 Krister Lagerstrom, et al. 
# Please see the file freevo/Docs/CREDITS for a complete list of authors.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MER-
# CHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# -----------------------------------------------------------------------

import re,os,glob,urllib,datetime,pickle,time
import config
import rssfeed

def convertDate(string):
    if not re.search("\d+\s+\S+\s+\d+",string):
       return datetime.date.today()
    itemDateList = re.split(" ", re.search("\d+\s+\S+\s+\d+",string).group())
    day=int(itemDateList[0])
    if itemDateList[1] == "Jan":
        month=1
    elif itemDateList[1] == "Feb":
        month=2
    elif itemDateList[1] == "Mar":
        month=3
    elif itemDateList[1] == "Apr":
        month=4
    elif itemDateList[1] == "May":
        month=5
    elif itemDateList[1] == "Jun":
        month=6
    elif itemDateList[1] == "Jul":
        month=7
    elif itemDateList[1] == "Aug":
        month=8
    elif itemDateList[1] == "Sep":
        month=9
    elif itemDateList[1] == "Oct":
        month=10
    elif itemDateList[1] == "Nov":
        month=11
    else:
        month=12
    year=int(itemDateList[2])
    itemDate = datetime.date(year,month,day)
    return itemDate

def checkForDup(string):
    cacheFile=config.FREEVO_CACHEDIR+"/rss.pickle"
    try:
       downloadedUrls=pickle.load(open(cacheFile,"r"))
    except IOError:
       return False
    except EOFError:
       return False
    foundFile=False
    for line in downloadedUrls:
        if string in line:
           foundFile=True
    return foundFile

def addFileToCache(string):
    cacheFile=config.FREEVO_CACHEDIR+"/rss.pickle"
    downloadedUrls=[]
    try:
       downloadedUrls = pickle.load(open(cacheFile,"r"))
    except IOError:
       pass
    downloadedUrls.append(string)
    pickle.dump(downloadedUrls, open(cacheFile,"w"))

def addFileToExpiration(string,goodUntil):
    ''' the new file gets added with the expiration date to the expiration file '''
    cacheFile=config.FREEVO_CACHEDIR+"/rss.expiration"
    downloadedFiles=[]
    try:
       downloadedFiles = pickle.load(open(cacheFile,"r"))
    except IOError:
       pass
    downloadedFiles.append(string + ";" + goodUntil.__str__())
    pickle.dump(downloadedFiles, open(cacheFile,"w"))

def checkForExpiration():
    ''' checking for expired files by reading the rss.expiration file the file
    contains the expiration date. once a file is expired it and its fxd gets
    deleted at the end the file gets removed from the rss.expiration file '''
    cacheFile=config.FREEVO_CACHEDIR+"/rss.expiration"
    try:
       downloadedFiles=pickle.load(open(cacheFile,"r"))
    except IOError:
       return 
    deletedItems = []
    for line in downloadedFiles:
        (filename,goodUntil)=re.split(";", line)
        expirationdate = datetime.datetime(*time.strptime(goodUntil, "%Y-%m-%d")[0:5])
        diff = expirationdate - datetime.datetime.today()
        if int(diff.days)<=0:
            deletedItems.append(line)
            tempList=re.split("\.",filename)
            fxdfile=tempList[0]
            for line in tempList[1:-1]:
                fxdfile=fxdfile+"."+line
            fxdfile=fxdfile+".fxd"
            try:
                os.remove(config.RSS_VIDEO+fxdfile)
            except OSError:
                print"removing the file %s failed"%(fxdfile)
            try:
                os.remove(config.RSS_VIDEO+filename)
            except OSError:
                print"removing the file %s failed"%(filename)
    for line in deletedItems:
#      try:
       downloadedFiles.remove(line)
#      except ValueError:
#          _debug_("removing the line %s failed" % (line))
    pickle.dump(downloadedFiles, open(cacheFile,"w"))

def createFxd(item,filename):
    if "audio" in item.type:
        fullFilename=config.RSS_AUDIO+filename
    else:
        fullFilename=config.RSS_VIDEO+filename
    tempList=re.split("\.",filename)
    ofile=tempList[0]
    for line in tempList[1:-1]:
        ofile=ofile+"."+line
    ofile=ofile+".fxd"
    try:
       file = open(ofile, 'w')
       file.write('<freevo>\n')
       file.write('   <movie title="%s">\n'%item.title)
       file.write('      <video>\n')
       file.write('         <file id="f1">%s</file>\n'%fullFilename)
       file.write('      </video>\n')
       file.write('      <info>\n')
       file.write('         <plot>%s</plot>\n'%item.description)
       file.write('      </info>\n')
       file.write('   </movie>\n')
       file.write('</freevo>\n')
       file.close()
    except IOError:
       print "ERROR: Unable to write FXD file %s"%(ofile)

def checkForUpdates():
    try:
        file = open(config.RSS_FEEDS,"r")
    except IOError:
       print "ERROR: Could not open configuration file %s"%(config.RSS_FEEDS)
       return

    for line in file:
        if line == '\n':
            continue
        if re.search("^#",line):
            continue
        try:
            (url,numberOfDays)=re.split(";", line)
        except ValueError:
            continue
        print "Check %s for updates" % url
        try:
            sock = urllib.urlopen(url)
            feedSource = sock.read()
            sock.close()
            for item in rssfeed.Feed(feedSource).items:
                diff = datetime.date.today() - convertDate(item.date)
                goodUntil = datetime.date.today() + datetime.timedelta(days=int(numberOfDays))
                if int(diff.days)<=int(numberOfDays) and not re.search("None",item.url):
                    if "audio" in item.type:
                        os.chdir(config.RSS_AUDIO)
                    else:
                        os.chdir(config.RSS_VIDEO)
                    filename=re.split("/",item.url).pop()
                    if (len(glob.glob(filename))==0) and not checkForDup(item.url):
                        if re.search("torrent",item.url):
                            print "Start downloading %s" % item.url
                            exitStatus=os.popen("bittorrent-console %s"%(item.url)).close()
                            filename=re.sub("\.torrent","",filename)
                        else:
                            print "Start downloading %s" % item.url
                            exitStatus=os.popen("wget %s" %(item.url)).close()
                        if not exitStatus:
                            createFxd(item,filename)
                            addFileToCache(item.url)
                            addFileToExpiration(filename,goodUntil)
                            print "Download completed (%s bytes)" % os.path.getsize(filename)
                        else:
                            print "Download failed - exit status %s." % exitStatus
                            os.popen("rm %s" %(filename))
        except IOError:
            print "ERROR: Unable to download %s. Connection may be down."%(url)

